import importlib
import os
import sys
from pathlib import Path

import autumn8  # systemwide import (to match user's annotations import) - not the local lib/code one
from click import ClickException

sys.path.append(os.getcwd())

PYTHON_FILE_EXTENSION = ".py"


def analyze_python_source(model_file_path, model_script_args=[]):
    # spec = importlib.util.spec_from_file_location("__main__", model_file_path)
    # module = importlib.util.module_from_spec(spec)
    # spec.loader.exec_module(module)
    args = sys.argv
    sys.argv = [args[0]]
    importlib.import_module(
        model_file_path[: -len(PYTHON_FILE_EXTENSION)]
    )  # TODO relative imports not working (fix: https://docs.python.org/3/library/importlib.html#importing-a-source-file-directly)
    sys.argv = [args, model_script_args]
    # exec(open(model_file_path).read(), {})
    models = autumn8.attached_models

    if len(models) < 1:
        # TODO try to manually find the model without autodl attachments
        raise ClickException(
            "The provided model file does not contain any models attached with `autumn8.attach_model`.\n"
            + "Please refer to our documentation to read on how to attached your models."
        )
    if len(models) > 1:
        print(
            "WARNING: Our current CLI implementation does not support multiple models in one file yet. Please contact us and file an inquiry to add this feature if you need it."
        )

    model, dummy_input, interns, externs, max_search_depth = models[-1]
    # idk, maybe there is better way to diffrentiate between tensorflow and pytorch source
    if "tensorflow" in sys.modules:
        import tensorflow as tf

        if isinstance(model, tf.keras.Model):
            model_file_bytes = autumn8.export_tensorflow_model_repr(
                model, dummy_input
            )
            return model_file_bytes, dummy_input, "TENSORFLOW"

    model_file_bytes = autumn8.export_pytorch_model_repr(
        model, dummy_input, interns, externs, max_search_depth
    )

    return model_file_bytes, dummy_input, "PYTORCH"


def load_tensorflow_protobuf(model_file_path):
    import tensorflow.compat.v1 as tf
    from google.protobuf.message import DecodeError
    from tensorflow.core.protobuf import saved_model_pb2
    from tensorflow.python.platform import gfile
    from tensorflow.python.util import compat

    graph_def = tf.GraphDef()

    with tf.Session() as sess:
        try:
            # this load method cannot load models from https://tfhub.dev/
            # crashes with `google.protobuf.message.DecodeError: Error parsing message with type 'tensorflow.GraphDef'`

            with tf.gfile.GFile(model_file_path, "rb") as file:
                graph_def = tf.GraphDef()
                graph_def.ParseFromString(file.read())

                return [n for n in graph_def.node]

        except DecodeError:
            # this load method cannot load models from @marcink
            # nasnet is missing input dimensions
            # posenet is missing meta_graphs - crashes with `IndexError: list index (0) out of range` on tf.import_graph_def(sm.meta_graphs[0].graph_def)
            with gfile.GFile(model_file_path, "rb") as file:
                data = compat.as_bytes(file.read())
                sm = saved_model_pb2.SavedModel()
                sm.ParseFromString(data)
                tf.import_graph_def(sm.meta_graphs[0].graph_def)
                return [n for n in sm.meta_graphs[0].graph_def.node]


def infer_tensorflow_protobuf_input_shape(model_file_path):
    graph_nodes = load_tensorflow_protobuf(model_file_path)
    inferred_batch_size, inferred_input_dims = (
        None,
        [],
    )

    for node in graph_nodes:
        if node.op == "Placeholder":
            shape_proto = node.attr["shape"]
            dim_sizes = [dim.size for dim in shape_proto.shape.dim]
            print("Detected input layer with shape", dim_sizes)
            dim_sizes += [None, None, None, None]
            dim_sizes = dim_sizes[0:4]
            dim_sizes = [None if size == -1 else size for size in dim_sizes]

            if len(shape_proto.shape.dim) > 0:
                [
                    inferred_batch_size,
                    *inputs,
                ] = dim_sizes
                if len(inputs) > 2:
                    inputs = [
                        inputs[-1],
                        *inputs[:-1],
                    ]
                inferred_input_dims.append(inputs)

    return (inferred_batch_size, inferred_input_dims)


def analyze_model_file(model_file_path, model_script_args=[]):
    extension = Path(model_file_path).suffixes[-1]

    (
        model_file,
        inferred_model_name,
        framework,
        inferred_quantization,
    ) = (None, None, None, None)
    _inferred_batch_size = None
    inferred_input_dims = []

    model_file = model_file_path
    inferred_model_name = Path(model_file_path).stem
    inferred_quantization = "FP32"

    if extension in [".py"]:
        model_file_bytes, dummy_input, framework = analyze_python_source(
            model_file_path, model_script_args
        )
        model_file_name = f"./{inferred_model_name}.adlpt"
        model_file = open(model_file_name, mode="wb")
        model_file.write(model_file_bytes.getbuffer())
        model_file.close()
        model_file = model_file_name

        [
            _inferred_batch_size,
            *inferred_input_dims,
        ] = dummy_input.shape

        inferred_input_dims = [inferred_input_dims]

    elif extension in [".mar"]:
        framework = "PYTORCH"
        # TODO analyze .mar file for input shape

    elif extension in [".pb", ".h5"]:
        framework = "TENSORFLOW"

        (
            _inferred_batch_size,
            inferred_input_dims,
        ) = infer_tensorflow_protobuf_input_shape(model_file_path)

    elif extension in [
        ".tflite"
    ]:  # TODO - add this extension also to the UI autofiller
        framework = "TFLITE"
        raise ClickException(f"Files with '{extension}' are not supported yet")

    elif extension in [".pt", ".pth"]:
        raise ClickException(
            f"Files with '{extension}' are not supported - please contact us at support@autumn8.ai to add the support."
        )

    else:
        raise ClickException(
            f"Files with '{extension}' are not supported - please contact us at support@autumn8.ai to add the support."
        )

    return (
        model_file,
        inferred_model_name,
        framework,
        inferred_quantization,
        inferred_input_dims,
    )



import autumn8
import numpy as np
import torch
from convblock import ConvBlock
from torch import nn


class TestModel(nn.Module):
    def __init__(self):
        super(TestModel, self).__init__()
        self.conv = torch.nn.Conv2d(3, 32, 3, 1)
        self.block1 = ConvBlock()
        self.block2 = ConvBlock()

    def forward(self, x):
        return self.block2(self.block1(self.conv(x)))


dummy_input = torch.randn(1, 3, 28, 28)
# just for testing prediction on server
np.save("inp.npy", dummy_input.numpy())
model = TestModel()

autumn8.attach_model(model, dummy_input)



import io
import json
import urllib

import requests
from requests.auth import HTTPBasicAuth

from autumn8.lib.api_creds import retrieve_api_creds
from autumn8.cli.cli_environment import CliEnvironment
from autumn8.common.config.s3 import init_s3


def url_with_params(url, params):
    url_parse = urllib.parse.urlparse(url)
    url_new_query = urllib.parse.urlencode(params)
    url_parse = url_parse._replace(query=url_new_query)

    new_url = urllib.parse.urlunparse(url_parse)
    return new_url


def require_ok_response(response):
    if response.status_code == 403:
        raise Exception(
            f"Received response {response.status_code}:\n{response.text}\n\nUser not authenticated; please run `autumn8-cli login` to authorize your CLI"
        )
    if response.status_code != 200:
        raise Exception(
            f"Received response {response.status_code}:\n{response.text}"
        )


def fetch_user_data(environment: CliEnvironment):
    user_id, api_key = None, None
    autodl_host = environment.value["host"]
    try:
        user_id, api_key = retrieve_api_creds()
    except:
        raise Exception(
            f"API key is missing! To configure API access, please visit {autodl_host}/profile and generate an API key, then run `autumn8-cli login`"
        )

    user_api_route = f"{autodl_host}/api/user"
    response = requests.get(
        user_api_route,
        headers={"Content-Type": "application/json"},
        auth=HTTPBasicAuth(user_id, api_key),
    )

    require_ok_response(response)
    return json.loads(response.text)["user"]


def post_model(environment, organization_id, model_config):
    autodl_host = environment.value["host"]
    api_route = f"{autodl_host}/api/lab/model"
    print("Submitting model to", api_route)
    response = requests.post(
        url_with_params(api_route, {"organization_id": organization_id}),
        headers={"Content-Type": "application/json"},
        data=json.dumps(model_config),
        auth=HTTPBasicAuth(*retrieve_api_creds()),
    )

    require_ok_response(response)
    return response.json()["id"]


def delete_model(environment, organization_id, model_id):
    autodl_host = environment.value["host"]
    new_url = url_with_params(
        f"{autodl_host}/api/lab/model",
        {"model_id": model_id, "organization_id": organization_id},
    )
    response = requests.delete(
        new_url,
        auth=HTTPBasicAuth(*retrieve_api_creds()),
    )
    require_ok_response(response)


def post_model_file(environment, bytes_or_filepath, s3_file_url):
    S3 = init_s3(environment.value["s3_host"])
    s3_bucket_name = environment.value["s3_bucket_name"]
    if isinstance(bytes_or_filepath, io.BytesIO):
        f = bytes_or_filepath
        S3.Bucket(s3_bucket_name).Object(s3_file_url).upload_fileobj(f)
    else:
        with open(bytes_or_filepath, "rb") as f:
            S3.Bucket(s3_bucket_name).Object(s3_file_url).upload_fileobj(f)


def async_prediction(environment, organization_id, model_id):
    autodl_host = environment.value["host"]
    new_url = url_with_params(
        f"{autodl_host}/api/lab/model/async_prediction",
        {
            "model_id": model_id,
            "organization_id": organization_id,
        },
    )
    response = requests.post(
        new_url,
        auth=HTTPBasicAuth(*retrieve_api_creds()),
    )
    require_ok_response(response)
    return response



import pkgutil
import random
import string
import sys
import tempfile
import traceback

import pkg_resources
import torch

# TODO that package resolver needs some work
env = dict(tuple(str(ws).split()) for ws in pkg_resources.working_set)

packages = []
for ws in pkgutil.iter_modules():
    if ws.ispkg:
        packages.append(ws.name)

system_packages = list(sys.builtin_module_names)
system_packages.append("os")
system_packages.append("pkg_resources")
for p in system_packages:
    env[p] = "0"
    packages.append(p)


def guess_if_package_is_external(name, interns, externs):
    split_name = name.split(".")
    for extern in externs:
        split_extern = extern.split(".")
        if split_extern[0] == split_name[0]:
            return True

    for intern in interns:
        split_intern = intern.split(".")
        if split_intern[0] == split_name[0]:
            return False

    return split_name[0] in env


# Max search depth is just some constant to ensure it will end.
# And if it is resolving that long, that can mean 2 thing:
# model has a lot of submodules importing submodules etc or
# we added some system module by accident and we are trying now recursively add some subsystems.
def package(model, interns=[], externs=[], max_search_depth=5):
    file = tempfile.NamedTemporaryFile()
    # NOTE: if any intern name will be equal to package_name, then packaged model will not be possible to import
    package_name_length = 10
    package_name = "".join(
        random.choices(string.ascii_lowercase, k=package_name_length)
    )
    resource_name = "model.pkl"

    success = False
    for i in range(0, max_search_depth):
        try:
            with torch.package.PackageExporter(file.name) as exp:
                for intern in interns:
                    exp.intern(intern)

                for extern in externs:
                    exp.extern(extern)
                exp.save_pickle(package_name, resource_name, model)

            success = True
            break
        except torch.package.package_exporter.PackagingError as e:
            for node in e.dependency_graph.nodes:
                if "error" in e.dependency_graph.nodes[node]:
                    if guess_if_package_is_external(node, interns, externs):
                        externs.append(node)
                    else:
                        interns.append(node)
        except:
            traceback.print_exc()
            break

    if not success:
        raise Exception(
            """Failed to resolve dependencies. Modules marked as intern:
{}

Modules marked as extern:
{}

If module was incorrectly labeled please pass it in autumn8.attach_model. Example:
autumn8.attach_model(model, input, interns = ["mymodule"], externs=["yaml"])
Specifing root of modules as extern should be enough
If all modules seems to be correctly marked as intern/extern please try bumping search depth. Example:
autumn8.attach_model(model, input, max_search_depth = 10)""".format(
                interns, externs
            )
        )
    requirements = {}

    for extern in externs:
        if extern in env and extern not in system_packages:
            requirements[extern] = env[extern]

    return file, requirements, package_name



import os

import boto3

AUTODL_S3_REGION = "us-east-1"


def init_s3(s3_host):
    # TODO - we need to somehow include these in CLI without hardcoding
    # AWS_ACCESS_KEY_ID = os.environ["NEXT_PUBLIC_AWS_ACCESS_KEY_ID"]
    # AWS_SECRET_ACCESS_KEY = os.environ["NEXT_PUBLIC_AWS_SECRET_KEY"]

    AWS_ACCESS_KEY_ID = "AKIASO72NKUYW7ONNRFO"
    AWS_SECRET_ACCESS_KEY = "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"

    return boto3.resource(
        "s3",
        region_name=AUTODL_S3_REGION,
        endpoint_url=s3_host,
        aws_access_key_id=AWS_ACCESS_KEY_ID,
        aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
    )


_S3 = None

# TODO flatten and reorganize our S3 structure
# right now, there's three places that attempt to determine the target bucket
# and all of these stack upon each other in the resulting url:
# - this endpoint already includes the bucket name, so the bucket is predetermined for this endpoint
# - when calling S3.Bucket(name), we also provide another bucket name for the url
# - finally, the bucket name is appended to the s3_file_url attribute somewhere during the upload
def get_s3_client():
    global _S3
    if _S3 is None:
        _S3 = init_s3(os.environ.get("NEXT_PUBLIC_AWS_S3_ENDPOINT"))

    return _S3



import questionary
from questionary import Choice

from autumn8.lib.api import fetch_user_data


def get_user_organizations(user_data):
    return [mem["organization"] for mem in user_data["memberships"]]


def pick_organization_id(environment):
    user_data = fetch_user_data(environment)
    user_organizations = get_user_organizations(user_data)

    organization_id = questionary.select(
        "Choose organization",
        choices=[
            Choice(title=f"{org['name']} ({org['id']})", value=org["id"])
            for org in user_organizations
        ],
        use_shortcuts=True,
    ).unsafe_ask()
    return organization_id


def verify_organization_id_access(environment, organization_id):
    user_data = fetch_user_data(environment)
    user_organization_ids = [
        org["id"] for org in get_user_organizations(user_data)
    ]
    if organization_id not in user_organization_ids:
        raise Exception(
            f"The user {user_data['email']} does not belong to the organization of id={organization_id}"
        )



import autumn8  # you need to `pip install autumn8` for this to work
import torch
import torch.nn.functional as F
from torch import nn


class Net(nn.Module):
    def __init__(self):
        super(Net, self).__init__()
        self.conv1 = nn.Conv2d(1, 32, 3, 1)
        self.relu1 = nn.ReLU()
        self.conv2 = nn.Conv2d(32, 64, 3, 1)
        self.maxPool = nn.MaxPool2d(2)
        self.dropout1 = nn.Dropout2d(0.25)
        self.flatten = nn.Flatten(1)
        self.dropout2 = nn.Dropout2d(0.5)
        self.fc1 = nn.Linear(9216, 128)
        self.relu2 = nn.ReLU()
        self.fc2 = nn.Linear(128, 10)

    def forward(self, x):
        x = self.conv1(x)
        x = self.relu1(x)
        x = self.conv2(x)
        x = self.maxPool(x)
        x = self.dropout1(x)
        x = self.flatten(x)
        x = self.fc1(x)
        x = self.relu2(x)
        x = self.dropout2(x)
        x = self.fc2(x)
        output = F.log_softmax(x, dim=1)
        return output


dummy_input = torch.randn(1, 1, 28, 28)
model = Net()
autumn8.attach_model(model, dummy_input)

# alternatively, to upload directly instead of using the CLI:
# autumn8.uploadModel("mnist", model, dummy_input)



import enum


# TODO remove - official CLI builds should only point toward production
class CliEnvironment(enum.Enum):
    LOCALHOST = {
        "host": "http://localhost",
        "s3_host": "http://localhost:4566",
        "s3_bucket_name": "predictor-bucket",
    }
    STAGING = {
        "host": "http://54.83.236.20",
        "s3_host": "https://autodl-staging.s3-accelerate.amazonaws.com",
        "s3_bucket_name": "autodl-staging",
    }
    PRODUCTION = {
        "host": "https://autodl.autumn8.ai",
        "s3_host": "https://autodl-staging.s3-accelerate.amazonaws.com",
        "s3_bucket_name": "autodl-production",
    }



import enum

class ServiceProvider(str, enum.Enum):
    AMAZON = "Amazon"
    GOOGLE = "Google Cloud Platform"
    ORACLE = "Oracle"
    AZURE = "Azure"
    

supported_hardwares = [
    {
        "name": "Ampere Altra",
        "vendor": "Ampere",
        "arch": "armv8.2+",
    },
    {
        "name": "Intel Xeon",
        "vendor": "Intel",
        "arch": "x86",
    },
    {
        "name": "Intel Xeon Platinum",
        "vendor": "Intel",
        "arch": "x86",
    },
    {
        "name": "Intel Xeon Platinum Ice Lake",
        "vendor": "Intel",
        "arch": "x86",
    },
    {
        "name": "Intel Cascade Lake CPU",
        "vendor": "Intel",
        "arch": "x86",
    },
    {
        "name": "Intel Skylake CPU",
        "vendor": "Intel",
        "arch": "x86",
    },
    {
        "name": "AMD EPYC",
        "vendor": "AMD",
        "arch": "amd64",
    },
    {
        "name": "Amazon Graviton2",
        "vendor": "Amazon",
        "arch": "Arm64 Neoverse",
    },
    {
        "name": "NVIDIA K80 GPUs",
        "vendor": "NVIDIA",
        "arch": "NVIDIA K80 GPUs",
    },
    {
        "name": "NVIDIA T4 GPUs",
        "vendor": "NVIDIA",
        "arch": "NVIDIA T4 GPUs",
    },
    {
        "name": "NVIDIA V100 GPUs",
        "vendor": "NVIDIA",
        "arch": "NVIDIA V100 GPUs",
    },
    {
        "name": "NVIDIA A10G",
        "vendor": "NVIDIA",
        "arch": "NVIDIA A10G",
    },
    {
        "name": "NVIDIA Tesla M60 GPU",
        "vendor": "NVIDIA",
        "arch": "NVIDIA Tesla M60 GPU",
    },
]


class Quantization(str, enum.Enum):
    FP32 = "FP32"
    FP16 = "FP16"
    INT8 = "INT8"


supported_quants = [q.value for q in Quantization]


class Framework(str, enum.Enum):
    TENSORFLOW = "TENSORFLOW"
    PYTORCH = "PYTORCH"
    TFLITE = "TFLITE"
    ONNX = "ONNX"


supported_frameworks = [f.value for f in Framework]

supported_domains = [
    {
        "domain": "Computer Vision",
        "tasks": [
            "Classification",
            "Object Detection",
            "Image Segmentation",
            "Depth Estimation",
            "Pose Estimation",
        ],
    },
    {"domain": "NLP", "tasks": ["Translation", "Text Classification", "BERT"]},
    {"domain": "Other", "tasks": ["Other"]},
]

# TODO make an enum out of this after I'm merged with Rafal's MLFlow PRs
def get_supported_providers():
    return [provider.value for provider in ServiceProvider]


supported_batch_sizes = [1, 8, 16, 32, 64]


class ModelFileType(str, enum.Enum):
    H5 = "h5"
    PB = "pb"
    FOLDER = "folder"
    MLFLOW = "mlflow"
    MAR = "mar"


def is_domain_task_supported(domain, task):
    for supported_domain in supported_domains:
        if supported_domain["domain"] == domain:
            for supported_task in supported_domain["tasks"]:
                if supported_task == task:
                    return True
    return False



import enum
import json
import random
import uuid
from configparser import NoOptionError, NoSectionError
import os

import click
import jsonschema
import questionary
from jsonschema import validate

from autumn8.lib import api, api_creds
from autumn8.lib import service as autodl
from autumn8.cli.cli_environment import CliEnvironment
from autumn8.cli.analyze import analyze_model_file
from autumn8.cli.examples import example_model_names
from autumn8.cli.interactive import (
    pick_organization_id,
    verify_organization_id_access,
)
from autumn8.common._version import __version__
from autumn8.common.config.settings import supported_quants

USER_ID_LENGTH = len(str(uuid.uuid4()))
API_KEY_LENGTH = 32

ENABLE_TOGGLEABLE_ENVIRONMENT = False


def use_environment_option(func):
    allowed_environments = (
        [env.name for env in CliEnvironment]
        if ENABLE_TOGGLEABLE_ENVIRONMENT
        else [CliEnvironment.PRODUCTION.name]
    )

    return click.option(
        "-e",
        "--environment",
        "--env",
        type=click.Choice(allowed_environments, case_sensitive=False),
        default=CliEnvironment.PRODUCTION.value,
        callback=lambda c, p, v: getattr(CliEnvironment, v),
        help="Environment to use",
        hidden=True,
    )(func)


@use_environment_option
@click.option(
    "-u",
    "--user_id",
    help="The ID of the user that the CLI will authenticate as in AutoDL.",
)
@click.option(
    "-a",
    "--api_key",
    help="API Key to use when authenticating in AutoDL from now on.",
)
def login(user_id, api_key, environment):
    """Store API credentials for the CLI to use in the future."""

    print(
        f"To setup up CLI access, please visit {environment.value['host']}/profile - once you're signed in, generate a new API Key, then copy and paste the API Key data from the browser here\ngg"
    )
    try:
        old_user_id, _ = api_creds.retrieve_api_creds()
        if old_user_id not in ["", None]:
            print(
                f"Warning: Replacing existing credentials for the user with id={old_user_id}"
            )
    except (NoSectionError, NoOptionError):
        pass

    # using unsafe_ask so that the script is properly aborted on ^C
    # (instead of questionary passing `None` as the user's prompt answer)
    if user_id is None or len(user_id) != USER_ID_LENGTH:
        user_id = questionary.text(
            "User ID",
            validate=lambda user_id: len(user_id) == USER_ID_LENGTH,
        ).unsafe_ask()
    else:
        print(f"User ID: {user_id}")
    if api_key is None or len(api_key) != API_KEY_LENGTH:
        api_key = questionary.text(
            "API Key",
            validate=lambda api_key: len(api_key) == API_KEY_LENGTH,
        ).unsafe_ask()
    else:
        print(f"API Key: {api_key}")

    api_creds.store_api_creds(user_id, api_key)
    user_data = api.fetch_user_data(environment)
    print(f"Credentials set up successfully for {user_data['email']}!")


INPUT_DIMS_JSONSCHEMA = {
    "type": "array",
    "minItems": 1,
    "items": {
        "type": "array",
        "minItems": 1,
        "items": {"type": "number"},
    },
}


def validate_input_dims_json(jsonString):
    if jsonString == "":
        return True

    try:
        jsonData = json.loads(jsonString)
        validate(instance=jsonData, schema=INPUT_DIMS_JSONSCHEMA)
    except (
        jsonschema.exceptions.ValidationError,
        json.decoder.JSONDecodeError,
    ):
        return False
    return True


INPUT_FILE_JSONSCHEMA = {
    "type": "array",
    "minItems": 1,
}


def validate_input_file(path):
    if not os.path.exists(path):
        return False

    try:
        with open(path, "r") as f:
            jsonData = json.load(f)

        validate(instance=jsonData, schema=INPUT_FILE_JSONSCHEMA)
    except (
        jsonschema.exceptions.ValidationError,
        json.decoder.JSONDecodeError,
    ):
        return False
    return True


def normalize_input_dims_for_api(input_dims):
    if not input_dims:
        return None

    inputs = json.loads(input_dims)
    inputs = [[str(dim) for dim in input] for input in inputs]
    return json.dumps(inputs, separators=(",", ":"))


# cannot use click prompt kwargs feature for the command options, because we
# want to infer input dimensions and the model name
def prompt_for_missing_model_info(
    model_name,
    quantization,
    input_dims,
    input_file,
    inferred_model_name,
    inferred_quantization,
    inferred_input_dims,
):
    # TODO - attempt reading model details from some configCache files
    if model_name is None:
        model_name = questionary.text(
            f"Please give a name to your model to be used in AutoDL (for example: '{random.choice(example_model_names)}')\n  Model name:",
            validate=lambda name: len(name) > 0,
            default=inferred_model_name,
        ).unsafe_ask()
    if quantization is None:
        quantization = questionary.select(
            "Choose quantization for the model",
            choices=supported_quants,
            use_shortcuts=True,
            default=inferred_quantization,
        ).unsafe_ask()

    class INPUT_METHODS(enum.Enum):
        FILE = "Upload input file"
        SHAPE = "Specify input shape"
        INFER = "Let us try to infer input shape"

    input_method = INPUT_METHODS.INFER.value

    if input_file is not None and input_dims is not None:
        print("Cannot specify both input file and input dimensions")
        input_file = None
        input_dims = None

    if input_dims is None and input_file is None:
        input_method = questionary.select(
            "Specify input method",
            choices=[method.value for method in INPUT_METHODS],
            use_shortcuts=True,
            default=INPUT_METHODS.INFER.value,
        ).unsafe_ask()

    if input_method == INPUT_METHODS.SHAPE.value and input_dims is None:
        input_dims = questionary.text(
            "Specify input dimensions for every model input as an array of JSON arrays "
            + '(ie. "[[3, 224, 224]]"), or leave blank to let us try to infer the input sizes")',
            validate=validate_input_dims_json,
            default=str(inferred_input_dims) if inferred_input_dims else "",
        ).unsafe_ask()

    if input_method == INPUT_METHODS.FILE.value and input_dims is None:
        input_file = questionary.text(
            "Provide an input file path (supported formats: .json)",
            validate=validate_input_file,
            default="",
        ).unsafe_ask()

    input_dims = normalize_input_dims_for_api(input_dims)

    return (model_name, quantization, input_dims, input_file)


@use_environment_option
@click.argument("model_file_path")
@click.argument("model_script_args", nargs=-1)
@click.option(
    "-n",
    "--name",
    "model_name",
    help="Name of the model to be used in AutoDL.",
)
@click.option(
    "-q",
    "--quantization",
    "--quants",
    type=click.Choice(supported_quants, case_sensitive=False),
    help="Quantization for the model.",
)
@click.option(
    "--input_dims",
    type=str,
    help="The model input dimensions, specified as a JSON array",
)
@click.option(
    "--input_file",
    type=str,
    help="The model input filepath",
)
@click.option(
    "-y",
    "--yes",
    "auto_confirm",
    is_flag=True,
    help="Skip all confirmation input from the user.",
)
@click.option(
    "-o",
    "--organization_id",
    "--org_id",
    type=int,
    help="The ID of the Organization to submit the model to",
)
@click.option(
    "-g",
    "--group_id",
    help="The ID of the group to submit the model to",
)
def submit_model(
    organization_id,
    model_file_path,
    model_script_args,
    model_name,
    auto_confirm,
    quantization,
    input_dims,
    input_file,
    group_id,
    **kwargs,
):
    """Submit a model to AutoDL."""

    # Priority: flags, then configCache, then inference, then interactive user input
    environment = kwargs["environment"]

    if organization_id is None:
        organization_id = pick_organization_id(environment)
    else:
        verify_organization_id_access(environment, organization_id)

    (
        model_file,
        inferred_model_name,
        framework,
        inferred_quantization,
        inferred_input_dims,
    ) = analyze_model_file(model_file_path, model_script_args)

    (
        model_name,
        quantization,
        input_dims,
        input_file,
    ) = prompt_for_missing_model_info(
        model_name,
        quantization,
        input_dims,
        input_file,
        inferred_model_name,
        inferred_quantization,
        inferred_input_dims,
    )
    model_config = {
        "name": model_name,
        "framework": framework,
        "quantization": quantization,
        "inputs": input_dims,
        "group_id": group_id,
    }

    print(
        "The details for your model are as follows:\n"
        + f"{json.dumps(model_config, indent=4)}\n"
    )

    if not auto_confirm:
        click.confirm(
            text="\n" + "Are you sure you want to upload that to AutoDL?",
            abort=True,
            default=True,
        )

    model_link = autodl.upload_model(
        environment, organization_id, model_config, model_file, input_file
    )
    print(f"\nDone!\nSee {model_link}")



import torch
from torch import nn


class ConvBlock(nn.Module):
    def __init__(self):
        super(ConvBlock, self).__init__()
        self.conv = nn.Conv2d(32, 32, 3, 1, padding=1)
        self.conv2 = nn.Conv2d(32, 32, 3, 1, padding=1)
        self.conv3 = nn.Conv2d(32, 32, 3, 1, padding=1)

    def forward(self, x):
        return self.conv3(self.conv2(self.conv(x))) + x



from typing import List

# scraped from https://modelzoo.co
# also see modelzoo-scrape.csv
example_model_names = """
OpenPose
Mask R-CNN
pytorch-CycleGAN-and-pix2pix
FastPhotoStyle
vid2vid
maskrcnn-benchmark
DCGAN-tensorflow
deep-image-prior
YOLO TensorFlow ++
Wavenet
PyTorch-GAN
faster-rcnn.pytorch
Deep Reinforcement Learning for Keras
StarGAN
pix2pixHD
albumentations
Augmentor
Open-source (MIT) Neural Machine Translation (NMT) System
Colornet
Image analogies
Deep Video Analytics
semantic-segmentation-pytorch
Chatbot
WassersteinGAN
MUSE
Faster-RCNN
Colorful Image Colorization
DQN-tensorflow
SRGAN
BigGAN-PyTorch
YOLO TensorFlow
pytorch-a2c-ppo-acktr
espnet
DeblurGAN
UNIT
TTS
Neural Sequence labeling model
UnsupervisedMT
faster rcnn
Lip Reading
waveglow
DeepOSM
deepvoice3_pytorch
deepspeech2
pytorch-seq2seq
pytorch-semantic-segmentation
async-RL
Popular Image Segmentation Models
ClassyVision
R-FCN
wgan-gp
sentiment-discovery
EDSR-PyTorch
pointnet.pytorch
DiscoGAN
neuraltalk2-pytorch
AnimeGAN
ChainerRL
Rainbow
Ultrasound nerve segmentation
NVIDIA/semantic-segmentation
chainer-DCGAN
pyannote-audio
loop
audio
Show, Attend and Tell
SLM-Lab
image-classification-mobile
Domain Transfer Network
GAN_stability
espresso
AdverTorch
Seq2seq-Chatbot
SSD
stackGAN-v2
pytorch-rl
DEXTR-PyTorch
jetson-reinforcement
official DiscoGAN implementation
MMDetection3D
Faster RCNN
self-critical.pytorch
seq2seq
RandWireNN
semi-supervised-pytorch
android-yolo
DCSCN Super Resolution
pytorch-maml-rl
speech
PyTorch-progressive_growing_of_gans
Neural Caption Generator
seq2seq.pytorch
cnn-vis
AttGAN
GAN-CLS
T2F
vnet.pytorch
Faster RCNN+Focal Loss
rl_a3c_pytorch
TensorFlow White Paper Notes
generative-models
pspnet-pytorch
vegans
pro_gan_pytorch
detecto
joint-vae
mushroom
nmtpytorch
U-Net
ARAE
gan-lib
MusicGenreClassification
mnist-svhn-transfer
lagom
piwise
/pytorch-SRResNet
vedaseg
pytorch-cpp-rl
brain-segmentation-pytorch
pytorch_RVAE
Speech Recognition
Improved Training of Wasserstein GANs
DeconvNet
prog_gans_pytorch_inference
neural-combinatorial-rl-pytorch
img_classification_pk_pytorch
opencv_transforms
PNASNet.pytorch
pix2pix-pytorch
pytorch-trpo(Hessian-vector product version)
DCGAN
**Faster_RCNN_for_DOTA**
AGE
Coupled Face Generation
CNN-LSTM-CTC
BEGAN-pytorch
Neural-IMage-Assessment
tacotron_pytorch
adversarial-autoencoder
samplernn-pytorch
Learning to Communicate with Deep Multi-Agent Reinforcement Learning
RetinaNet
Open Source Chatbot with PyTorch
stitchfix/fauxtograph
Neural-IMage-Assessment 2
nonauto-nmt
torch_waveglow
DeepMask object segmentation
Tacotron-pytorch
GAN-weight-norm
DiffAI
Seg-Uncertainty
deep_image_prior
pytorch-seq2seq-intent-parsing
universal-triggers
Deep-Image-Analogy-PyTorch
Pytorch-DPPO
**MobileNetV2**(TVM Supported)
SVHNClassifier
Aspect-level-sentiment
DAGAN
captionGen
MXNMT
neuron-selectivity-transfer
Mnemonic Descent Method
pytorch-trpo
chainer_examples
bandit-nmt
paysage
adversarial-patch
back2future.pytorch
Image Embedding Learning
wasserstein-gan
Categorical DQN
pytorch_image_classifier
biogans
famos
Improved CycleGAN
RobustBench
MobileNetV2
Pytorch Poetry Generation
**deepspeech**
Unsupervised Object Counting
sparse-structure-selection
Fast Neural Style for Image Style Transform by Pytorch
translagent
vae_vpflows
NVIDIA/unsupervised-video-interpolation
im2im
tgan
chainer_caption_generation
mmd/jmmd/adaBN
google_evolution
mxnet-seq2seq
chainer-dfi
LSGAN
bytenet
AVO-pytorch
ppo_pytorch_cpp
Adaptive-segmentation-mask-attack (ASMA)
Monolingual and Multilingual Image Captioning
Hierarchical Attention Network for Document Classification
mxnet-audio
AEGeAN
imagenet-vgg
improved-gan
chainer-Variational-AutoEncoder
VAE
multilabel
chainer-gan-denoising-feature-matching
**dspnet**
SqeezeNet
AdversarialAutoEncoder
ADDA
SEC
chainer_encoder_decoder
MXSeq2Seq
Ultrasound nerve segmentation
FocalLoss(CUDA)
LSGAN
multi-task
VGAN Tensorflow
L-GM-Loss
chainer_superresolution
Image colorization
Neural Image Caption
unrolled-gan
retrieval chatbot
began
SRCNN
mxnet_kaldi
text2image
chainer-srcnn
SSD+FPN
translatR
Hierarchical Question-Imagee Co-Attention
chainer-gan-trainer
AdversarialCrypto
AdversarialText
AttentionOCR
Audioset
BrainCoder
CognitiveMappinGANPlanning
Compression
DeepSpeech
DeepLab
DELF
GAN
Im2txt
LearningUnsupervisedLearning
LFADS
MaskGAN
ObjectDetection
PCLRL
SLIM
Street
Vid2depth
StyleGAN
Policy Gradient
SegNet and Bayesian SegNet
ResNets
Deep Hand
Inception-BN full ImageNet model
DeepYeast
BVLC AlexNet
BVLC Reference CaffeNet
Detectron
ResNet-50
pytorch-kaldi
detectron2
MMDetection
MMSegmentation
MMEditing
tensorboard-pytorch
gandissect
tensorwatch
rlpyt
PyTorch-VAE
ray
InfoGAN
pix2pix
Colorful Image colorization
Auxiliary Classifier GAN
Adversarial Autoencoder
Bidirectional GAN
Boundary-Seeking GAN
Conditional GAN
Context-Conditional GAN
Coupled GANs
CycleGAN
Deep Convolutional GAN
DiscoGAN
DualGAN
Generative Adversarial Network
InfoGAN
LSGAN
Pix2Pix
PixelDA
Semi-Supervised GAN
Super-Resolution GAN
Wasserstein GAN
Wasserstein GAN GP
Adversarial Auto-encoders
DCGAN
DCGAN face generation
Variational Autoencoder with deconvolutions
t-SNE of image CNN fc7 activations
pytorch-pretrained-BERT
Magenta
Kubeflow
Kaldi
flair
AllenNLP
Faster R-CNN
fairseq-py
pyro
pytext
**InsightFace**
DensePose
Sentence Classification with CNN
Neural Style
pytorch vision
Realtime Multi-Person Pose Estimation
denseNet
CapsNet-Tensorflow
Reading Wikipedia to Answer Open-Domain Questions
face-alignment
AlphaPose
Tensorflow-Project-Template
Striving for Simplicity
BERT-PyTorch
Minigo
Single Shot MultiBox Detector
**Faster RCNN+Deeplab+R-FCN+Deformable-ConvNets+FPN+SoftNMS**
pythia
OpenPose TensorFlow
Fast R-CNN
Detectron.pytorch
yolov3
DeepJazz
Neural Style
CapsNet-Keras
CIFAR-10 on Pytorch with VGG, ResNet and DenseNet
Transformer-XL
3DDFA
LASER
pytorch text
TCN
pygcn
SqueezeNet
pytorch-generative-model-collections
Mask R-CNN
InferSent
XLM
pytorch-playground
neuralcoref
Person-reID_pytorch
Neural Style Transfer
FCIS
Averaged Stochastic Gradient Descent with Weight Dropped LSTM
FlowNet 2.0
PyTorch-NLP
style-transfer
kornia
pytorch-faster-rcnn
Data Augmentation and Sampling for Pytorch
CRF-RNN
ChainerCV
YOLO2
end-to-end-negotiator
Pretty Tensor
poincare-embeddings
tacotron2
Center Loss
pytorch-openai-transformer-lm
Deep Feature Flow
pytorch-qrnn
efficient_densenet_pytorch
Single Shot MultiBox Detector
attention-transfer
higher
Neural Turing Machine in TensorFlow
hub
Dilated ResNet combination with Dilated Convolutions
deep-head-pose
MinkowskiEngine
**sockeye**
TensorNets
Human Pose Estimation with TensorFlow
CapsGNN
inplace_abn
pytorch-retinanet
pytorch-yolo2
Realtime_Multi-Person_Pose_Estimation
SparseConvNet
redner
pytorch-fcn
Mask-RCNN
torchstat
pytorch-pose
Video Frame Interpolation via Adaptive Separable Convolution
pytorch-toolbelt
tsn-pytorch
pytorch-struct
Cnn-text classification
jiant
RepDistiller
cvxpylayers
grad-cam
pytorch-deform-conv
bindsnet
yusuketomoto/chainer-fast-neuralstyle
torchMoji
BERT-NER
LM-LSTM-CRF
CNN visualization using Tensorflow
probtorch
Character-Aware Neural Language Models
pytorch-pruning
medicaltorch
MTCNN (original detector)
pytorch-wavenet
SfmLearner-Pytorch
pytorch - fid
Generative Handwriting Demo using TensorFlow
CrypTen
densenet
CondenseNet
FaceBoxes.PyTorch
translate
PyTorch-Style-Transfer
PVANet
voicefilter
bottom-up-attention-vqa
Flow-Guided Feature Aggregation
NCRF
pytorch2keras
pytorch-cnn-finetune
Convolutional LSTM Network
texar-pytorch
sphereface_pytorch
extension-cpp
euclidesdb
MTCNN
pytorch-capsule
breast_cancer_classifier
flownet
gpt-2-Pytorch
graphsage-simple
vqa.pytorch
torchbearer
pytorch-deeplab-resnet
DiracNets
Wide ResNet model in PyTorch
illustration2vec
DUC
MXNet Face
Deep-Compression-AlexNet
DPN
YOLO in caffe
torchgpipe
pytorch2caffe
FlashTorch
pytorch-pose-hg-3d
ResNet
matchbox
ClusterGCN
NeuralArt
convert_torch_to_pytorch
Random-Erasing
Visual question answering
attn2d
RoIAlign.pytorch
STN OCR
torchbeast
treelstm.pytorch
MMPose
SMASH
torchprof
Structured-Self-Attention
qp solver
Arnold
ConvE
pytorch-ntm
ban-vqa
pytorch-dense-correspondence
pytorch-maml
CapsNet-pytorch
pytorch-made
MMAction2
colorization-pytorch
trellisnet
yolo2-pytorch
pytorch-coviar
pytorch-flows
pytoune
Poutyne
ResNeXt.pytorch
STEAL
A Structured Self-Attentive Sentence Embedding
FlappyBird DQN
deformable-convolution-pytorch
Xlearn
pixyz
honk
Shufflenet-v2-Pytorch
DrQA
GraphWaveletNeuralNetwork
DeepPose
wavetorch
densebody_pytorch
geoopt
lightly
Pytorch-contrib
torch2coreml
optnet
fast-neural-style
NGCN
NGCN
Weakly_detector
magnet
BMXNet
ggnn.pytorch
tbd-nets
YOLOv2
vsepp
CoordConv-pytorch
deepfloat
pytorch-i-revnet
QANet-pytorch
ResNet-38
TripletLoss(FaceNet)
SimGNN
PyTorch-LBFGS
pyvarinf
pywick
pytorch_cluster
gogh
pytorch-es
PytorchNeuralStyleTransfer
R2Plus1D-PyTorch
gensen
generative-query-network-pytorch
YellowFin_Pytorch
Face_Attention_Network
nonechucks
Tensorflow FastText
pytorch_fft
Pytorch Geometric Temporal
transfer-nlp
Snapshot Ensembles
faster-rcnn
HMM in TensorFlow
OpenChem
RetinaNet
pyscatwave
GradientEpisodicMemory
quick-nlp
pytorch-explain-black-box
graph_convnets_pytorch
pytorch-dqn
Learning to learn by gradient descent by gradient descent
ClariNet
vel
GrouPy
neural-art-mini
pyinn
pytorch-dnc
nmp_qc
AttentionWalk
NLP-Caffe
dnc
webdataset
pytorch-sgns
caffemodel2pytorch
glow-pytorch
context_encoder_pytorch
neural-style-pt
MTCNN_face_detection_alignment
salad
WideResNets
Neural programmer-interpreter
YOLO/YOLOv2
SFD_pytorch
relational-rnn-pytorch
PyramidNet-PyTorch
inferno
Mask R-CNN
LearningToCompare-Pytorch
Pytorch-Toolbox
fusenet
flambe
pytorch-hessian-eigenthings
DeepRL-Grounding
pytorch-tools
FewShotLearning
GAM
APPNP
dni-pytorch
Skip-Thought Vectors
DQN
TuckER
Lucent
VIN_PyTorch_Visdom
pydlt
Visual Search
fcn
ORN
PyTorch Realtime Multi-Person Pose Estimation
ChainerMN
torchani
DQN-chainer
AccSGD
DQN
Character CNN
FreezeOut
pytorch-transformer
pytorch_compact_bilinear_pooling v1
PyTorchWavelets
beauty-net
Pytorch-NCE
VGG-CAM
NALU-pytorch
Optical Flow Estimation using a Spatial Pyramid Network
pt-dilate-rnn
pywarm
sparktorch
HCN-pytorch
VRNN
pytorch-zssr
mpl.pytorch
packnet
DCC
C3D
volksdep
dockerface
pytorch-text-recognition
odin-pytorch
SGCN
DRRN-pytorch
Splitter
chainer-char-rnn
pytorch-retraining
SEAL-CI
OpenFacePytorch
vqa-winner-cvprw-2017
ChainerUI
simple-effective-text-matching-pytorch
torch-two-sample
DeepCORAL
convnet-aig
fmpytorch
hessian
DGC-Net
Real-time style transfer
SENet
crnn-mxnet-chinese-text-recognition
UntrimmedNets
caffe_to_torch_to_pytorch
mctorch
MobileNet
Attentive Object Tracking
fenchel-young-losses
FractalNet
Pytorch-Sketch-RNN
Attentive Recurrent Comparators
QuCumber
pytorch-pose-estimation
skip-gram-pytorch
FCN-pytorch-easiest
CompactBilinearPooling-Pytorch v2
StackNN
bigBatch
piggyback
Smile detection with a CNN
FaceDetection-ConvNet-3D
DeepPose TensorFlow
neural style transfer
FCNT
e2e-model-learning
cifar10
GoogleNet-V2
AOGNet
pytorch-prunes
transducer
DeepBox
pix2pix
CNN-LSTM-CTC
deep-forecast-pytorch
logger
Realtime Multi-Person Pose Estimation
deep-auto-punctuation
DenseNet
pytorch-sift
doomnet
binary-wide-resnet
DSSD+TDM
pytorch_NEG_loss
Draw like Bob Ross
DepthNet
NoisyNaturalGradient
pytorch-smoothgrad
SRDenseNet-pytorch
NALU
macarico
PySNN
RNN-Transducer
EigenDamage-Pytorch
mixup_pytorch
Probabilistic Programming and Statistical Inference in PyTorch
A3C-PyTorch
lang-emerge-parlai
nmn-pytorch
MSG-Net
dni
distance-encoding
Head Pose
meProp
ewc.pytorch
minimal_glo
binary_net
TDD
PyTorch_GBW_LM
Torchelie
interaction_network_pytorch
mixup
Tiny Face (original detector)
rl-multishot-reid
wavenet
deep-dream-in-pytorch
Whale Detector
pytorch-extension
SSD Text Detection
pytorch-NeuCom
pytorch-mcn
FFTNet
NASNet-A-Mobile. Ported weights
integrated-gradient-pytorch
mxnet-model
PSPNet
E2FAR
EPSR
Neural-Style-MMD
Ladder Network
pytorch-fitmodule
cogitare
**CTPN.mxnet**
imsat
molencoder
scalingscattering
pytorch_TDNN
nninit
sqeezenet
quantile-regression-dqn-pytorch
torchcraft-py
kaggle_CIFAR10
LOLA_DiCE
SINE
forward-thinking-pytorch
CNN Based Text Classification
graph-cnn
chainer-ssd
pytorch_Highway
pytorch-nec
Factorized-Bilinear-Network
candlegp
aorun
DRQN
Faster RCNN
DAGGER
Char-RNN
Highway Network
shampoo.pytorch
pnn.pytorch
chainer-visualization
neural-dream
**Gluon Dynamic-batching**
chainer-ResNet
Tor10
chainer-gogh
binary-stochastic-neurons
mia
Yelp Restaurant Photo Classifacation
ko_en_NMT
psmm
dpwa
MXNet for CTR
generative_zoo
pytorch extras
DarkRank
crnn
LSTNet
fast-rcnn
cat-net
CRU-Net
CortexNet
crnn (with Chinese Support)
CDL
Deep-Leafsnap
reseg-pytorch
diffdist
TRPO
mxnet-videoio
ram
MalConv-Pytorch
fluidnet_cxx
osqpth
simple-fast-rnn
ShuffleNet
segnet
mlpnlp-nmt
segnet
FacialLandmark
chainer-ResDrop
AlexNet3D
StrassenNets
chainer-DenseNet
proxprop
pytorch-ctc
siamese
pyTorch_NCE
Character-level CNN Text Classification
e2c-pytorch
flow
pytorch_hmax
LSTM to predict gender of a name
torch-metrics
Torchlite
netharn
eve.pytorch
YOLO
U-Net
chainer_stylenet
CapsNet
pt-styletransfer
zalando-pytorch
poisson-convolution-sum
PyTorch-docset
neural-assembly-compiler
pytorch-cns
svm
OrthNet
VanillaCNN
Convolutional Neural Fabrics
lgamma
NonLocal+SENet
SSD+Focal Loss
Deeplab v2
OpenPose
sentiment-analysis
Traffic sign classification
galapagos_nao
variational-autoencoder
AC-BLSTM
anuvada
Xception
SqueezeDet
wavenet
pyprob
DeepHumanPrediction
Sequential Matching Network
Visual-Semantic Embedding
YOLO/dark2mxnet
mx-pointnet
DEF
conv-vis
U-Net(kaggle dstl)
PointCNN.MX
VQA
Tiny Face (w/ training)
siamese
Language/dialect identification with multiple character-level CNNs
SDPoint
Zero-shot Intent CapsNet
P3D
SpectralLDA
memnn
chainer-example-overfeat-classify
YOLOtiny_v2
LightCNN
cnn+Highway Net
cascade-rcnn-gluon
MemN2N
deformable-conv
Tacotron
st-resnet
siamese_network_on_omniglot
mxnet-recommender
chainer_ca
torchplus
Xception+Keras2MXNet
VisualBackProp
Range Loss
RangeLoss
fast-style-transfer
chainer_rnnlm
vat
SuperResolutionCNN
Chest-XRay
SENet(from Caffe)
Convolutional Sketch Inversion
parserChiang
NER with Bidirectional LSTM-CNNs
PPO
prednet
adgm
Convolutional Pose Machines
Dynamic pose estimation
Reinspect
NetVlad
IMSAT
LUCAD
openai-mxnet
mnist-oneshot
VQA
geometric-matching
collaborative_filtering
chainer-fluid
ddgm
delira
FractalNet
IOULoss
cicada classification
DPSH
MemN2N
Neural Programmer-Interpreters
sequence-sampling
minibatch_discrimination
Autoencoder
DeepContextualBandits
DifferentialPrivacy
DomainAdaptation
Inception
KeypointNet
LearningToRememberRareEvents
LexNetc
LM1B
LmCommonsense
Namignizer
NeuralGpu
NeuralProgrammer
NextFramePrediction
PTN
MARCO
QAKG
RealNvp
REBAR
ResNet
Seq2species
SkipThoughts
Swivel
SyntaxNet
TCN
Textsum
Transformer
VideoPrediction
Deep Q-Network
AC
A3C
Network in Network model
Places-CNN model from MIT.
GoogLeNet GPU implementation from Princeton.
ParseNet
Holistically-Nested Edge Detection
VGG Face CNN descriptor
Yearbook Photo Dating
CCNN
Faster R-CNN
Sequence to Sequence - Video to Text
Pascal VOC 2012 Multilabel Classification Model
SqueezeNet
Mixture DCNN
Mulimodal Compact Bilinear Pooling for VQA
Neural Activation Constellations
ResFace101
Striving for Simplicity
VGG 4x without degradation
Using Ranking-CNN for Age Estimation
SSD
BVLC GoogLeNet
BVLC Reference RCNN ILSVRC13
DenseNet-121
Inception v1
SqueezeNet
ZFNet-512
NeMo
transformers
reformer-pytorch
TorchCV
facenet-pytorch
pytorch3d
botorch
pretrained-models.pytorch
gpytorch
spotlight
inferno-sklearn
PyTorch-Encoding
skorch
ignite
simple-faster-rcnn-pytorch
pytorchviz
pytorch-summary
apex
ELF
pytorch_geometric
fastai
gpytorch
QNNPACK
torchdiffeq
dgl
torchgeometry
AdaBound
pytorch-OpCounter
Catalyst
Ax
pytorch-lightning
learn2learn
kaolin
pytorch-metric-learning
pytorch-optimizer
EfficientNet PyTorch
PyTorch/XLA
NVlabs/DG-Net
pulse
Self-Norm Nets
LSTM for HAR
pix2pix
DIRNet
DFI
Context Encoder
chainer_examples
qrnn
chainer-libDNN
Face68Pts
VGG16 Deconvolution network
mxnet_tetris
**gluon-nlp**
SegNet
VGG 16 (with pre-trained weights)
VGG 19 (with pre-trained weights)
DQN
GluonSeg
Double DQN
matchnet
DeepID v1
C3D
DRL
cnnbilstm
NASNet-A
FCN-ASPP
Deep-Q learning Pong with TensorFlow and PyGame
DeepLab
VGG-Face
LRCN
SRCNN
DeepDrive
Berkeley DeepDrive
Princeton deepdriving
Sentiment Analysis
AlexNet
Inception v2
StyleTransfer
VGG19
Multi-GPU Multi-Label DenseNet
Light-Head R-CNN
Segnet
EAST
RC3D
Receptive Field Tool
LSTM on the IMDB dataset (text sentiment classification)
Bidirectional LSTM on the IMDB dataset
1D CNN on the IMDB dataset
1D CNN-LSTM on the IMDB dataset
LSTM-based network on the bAbI dataset
Memory network on the bAbI dataset (reading comprehension question answering)
LSTM text generation
Using pre-trained word embeddings
FastText on the IMDB dataset
Simple CNN on MNIST
Simple CNN on CIFAR10 with data augmentation
Inception v3
Neural Style Transfer
Deep dreams
Stateful LSTM
Siamese network
Pretraining on a different dataset
s2cnn
""".split(
    "\n"
)



import io
import os
import uuid
from pathlib import Path
from typing import Optional

from autumn8.lib import api


def upload_model(
    environment,
    organization_id,
    model_config,
    model_file: str,
    input_file_path: Optional[str],
):
    autodl_host = environment.value["host"]
    if type(model_file) == io.BytesIO:
        model_file.seek(0)
        model_file_name = model_config["name"]  # TODO add extension?
    else:
        model_file_name = os.path.basename(model_file)

    s3_file_url = (
        f"autodl-staging/models/{organization_id}-{uuid.uuid4()}"
        + f"-{model_file_name}"
    )

    print("Uploading the model files...")
    api.post_model_file(environment, model_file, s3_file_url)
    model_config["s3_file_url"] = s3_file_url

    if input_file_path != None and len(input_file_path) > 0:
        filename = Path(input_file_path).name
        s3_input_file_url = (
            f"autodl-staging/inputs/{organization_id}-{uuid.uuid4()}-{filename}"
        )
        print("Uploading the input files...")
        api.post_model_file(environment, input_file_path, s3_input_file_url)
        model_config["s3_input_file_url"] = s3_input_file_url

    print("Creating the model entry in AutoDL...")
    model_id = api.post_model(environment, organization_id, model_config)

    print("Starting up performance predictor...")
    api.async_prediction(environment, organization_id, model_id)
    return f"{autodl_host}/{organization_id}/performancePredictor/dashboard/{model_id}"



import io
import json
import os
import pickle
import sys
import zipfile

import torch

from autumn8.lib.package_resolver import package
from autumn8.common._version import __version__

sys.path.append(os.path.join(os.path.dirname(__file__), "../"))


def write_model(model, dummy_input, bytes):
    traced = torch.jit.trace(model, dummy_input)
    traced.save(bytes)


def export_tensorflow_model_repr(
    model, dummy_input, interns=[], externs=[], max_search_depth=5
):
    return export_model_repr(
        model, dummy_input, "TENSORFLOW", interns, externs, max_search_depth
    )


def export_pytorch_model_repr(
    model, dummy_input, interns=[], externs=[], max_search_depth=5
):
    return export_model_repr(
        model, dummy_input, "PYTORCH", interns, externs, max_search_depth
    )


def export_model_repr(
    model, dummy_input, framework, interns=[], externs=[], max_search_depth=5
):
    bytes = io.BytesIO()

    if not isinstance(dummy_input, tuple):
        dummy_input = (dummy_input,)

    if framework == "PYTORCH":
        externs.append("torch")
        externs.append("torchvision")

    if framework == "TENSORFLOW":
        externs.append("tensorflow")

    file, requirements, package_name = package(
        model, interns, externs, max_search_depth
    )

    with zipfile.ZipFile(bytes, "w") as zip:
        zip.writestr(
            "MANIFEST",
            json.dumps(
                {
                    "version": __version__,
                    "package_name": package_name,
                }
            ),
        )
        zip.write(file.name, arcname="model.package")
        requirement_list = []
        for package_name, package_version in requirements.items():
            requirement_list.append(f"{package_name}=={package_version}")

        zip.writestr("requirements.txt", "\n".join(requirement_list))

    bytes.seek(0)
    return bytes


def load_model(filename):
    with zipfile.ZipFile(filename) as z:
        with z.open("MANIFEST", "r") as manifest:
            package_name = json.loads(manifest.read())["package_name"]
        with z.open("model.package", "r") as model_file:
            resource_name = "model.pkl"
            imp = torch.package.PackageImporter(model_file)
            loaded_model = imp.load_pickle(package_name, resource_name)
            return loaded_model


attached_models = []


# TODO: create 'annotate' module?
def attach_model(
    model, example_input, interns=[], externs=[], max_search_depth=5
):
    attached_models.append(
        (model, example_input, interns, externs, max_search_depth)
    )



from pathlib import Path

from autumn8.common._version import __version__
from setuptools import find_namespace_packages, setup

this_directory = Path(__file__).parent
readme_content = (this_directory / "README.md").read_text()

setup(
    name="autumn8",
    version=__version__,
    author="Autumn8",
    author_email="marcink@radcode.co",  # TODO use some common support mail from autumn8?
    packages=find_namespace_packages(include=["autumn8.*"]),
    description="Utilities to export models to the autumn8.ai service",
    long_description=readme_content,
    long_description_content_type="text/markdown; charset=UTF-8; variant=GFM",
    entry_points={
        "console_scripts": [
            "autumn8-cli=autumn8.cli.main:main",
        ],
    },
)



import click

from autumn8.cli.commands import login, submit_model, use_environment_option
from autumn8.cli.interactive import fetch_user_data
from autumn8.common._version import __version__


@use_environment_option
def test_connection(environment):
    """
    Test connection to the autumn8.ai service, using the configured API key.
    Displays the user's email address upon successful connection.
    """
    user_data = fetch_user_data(environment)
    print(f"Hello! You're authenticated as {user_data['email']}")


@click.group()
@click.version_option(version=__version__)
def main():
    pass


main.command()(submit_model)
main.command()(test_connection)
main.command()(login)

if __name__ == "__main__":
    main()
